local string, type, tostring, pairs, table, tonumber = string, type, tostring, pairs, table, tonumber

module('nsg')

-- there are another GOOD free utf8 lib in internet google for 'AKFAvatar utf8' 
-- but this lib is enough now

--- Return byte offset of k-utf8char
-- @param s utf-8 string
-- @param k number

utf8index = function (s, k)
  local b, n = 0, 0
  local over = #s + 1
  for i = 1, k do
    b, n = s:find("([%z\1-\127\194-\244][\128-\191]*)", n + 1)
    if not n then break end
  end
  return b or over, n or over
end

--- Return count of printable symbols include spaces
-- @param s utf-8 string
utf8len = function(s)
                   local _, len = s:gsub("[^\128-\193]", "")
                   return len
                end

--- Return reversed version of input string
-- @param s utf-8 string
utf8reverse = function(s)
                       local rev=""
                       for uchar in s:gfind("([%z\1-\127\194-\244][\128-\191]*)") do
                          rev=uchar..rev
                       end
                       return rev
                    end

--- Return substring of input string, like string.sub but using utf-8 instead of ASCII
-- @param s utf-8 string
-- @param st start of substring in UTF8
-- @param en end of substring


--[[
-- version of IMosunov
utf8sub = function(str, b, e)
  local x, y
  if e then
    x, y = utf8index(str, e)
  end
  return str:sub(utf8index(str, b), y)
end
--]]

utf8sub = function(s, st, en)
         local rst, ren = 1, 1
         local len = utf8len(s)
         st = st<0 and len+st or st
         en = en and (en<0 and len+en or en) or len
         st = st<1 and 1 or st
         en = en>len and len or en
         if st>len then return "" end
         if en<1 then return "" end
         if st>en then return "" end

         local ret={}
         local i=0
         for uchar in s:gmatch("([%z\1-\127\194-\244][\128-\191]*)") do
            i=i+1
            if i<st then
               rst = rst + #uchar
            end
            if i<=en then
               ren = ren + #uchar
            end

            if i>=en then
               return (s:sub(rst, ren-1))
            end
         end
         return ""
      end

--- Fast(then sub) return substring of input string, using initial byte offset and  utf-8 length
-- @param s utf-8 string
-- @param bst start of substring in BYTES
-- @param ulen length of UTF8-encoded substring in UTF8-chars

utf8byteSub = function(s, bst, ulen)
             if bst>#s then return "" end
             if ulen<1 then return "" end
             local uchar
             local pos = bst
             repeat
                uchar =  (s:match("([%z\1-\127\194-\244][\128-\191]*)", pos))
                if uchar then
                   ulen = ulen - 1
                   pos = pos + #uchar
                end
                if (ulen == 0) or (not uchar) then
                   return s:sub(bst, pos-1)
                end
             until not uchar
             return ""
          end

--- Return !!! BYTE offset NOT utf8 of pattern in input string
-- @param s utf-8 string
-- @param p search pattern
-- @param i initial utf8 offset
utf8find = function(s,p,i)
                    local len = utf8len(s)
                    i = i<0 and len+i or i
                    i = i<1 and 1 or i
                    if i> len then return nil end
                    local sEnd = utf8sub(s,i)
                    local sSta = utf8sub(s,1,i-1)
                    local a,b,c = sEnd:find(p)
                    return (a and (a+#sSta) or nil),(b and (b+#sSta) or nil),c
                 end

--- Return splited  utf8 string
-- @param s utf-8 string
-- @param u8len len of substring in utf8 chars or 80 by default

utf8split =
   function (s, u8len)
      u8len = (tonumber(u8len) or -1) >0 and u8len or 80
      s = s or ""
      local p = 1
      local out = {}
      local i = u8len or 80
      while p<=#s do
         out[#out+1] = utf8byteSub(s, p, i)
         p = p + #out[#out]
      end
      return out
   end

function luaToHumanUTF8(s)
   if type(s)=='string' then
      if #s>1 and s:sub(1,1)=='"' and s:sub(-1)=='"' then
	 s = s:sub(2,-2)
      end
      return s:
      gsub('\\(["\'])','%1'):
      gsub('\\n','\226\144\138'):
      gsub('\\r','\226\144\141'):
      gsub('\\t','\226\144\137'):
      gsub('\\127','\226\144\161'):
      gsub('\\([0-9]+)',
	   function(num)
	      return '\226\144'..string.char(128+tonumber(num))
	   end
	)
   else
      return s
   end

end
